/*
 *
 *  This file ist part of the software MonaLisa.
 *  MonaLisa is free software, dependend on non-free software. For more information read LICENCE and README.
 *
 *  (c) Molekulare Bioinformatik, Goethe University Frankfurt, Frankfurt am Main, Germany
 *
 */

package monalisa.data.output;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

import monalisa.Project;
import monalisa.data.pn.TInvariant;
import monalisa.results.Configuration;
import monalisa.results.DistanceMatrix;
import monalisa.results.DistanceMatrixConfiguration;
import monalisa.results.TInvariants;
import monalisa.results.TInvariantsConfiguration;
import monalisa.tools.tinv.TInvariantTool;

public class DistanceMatrixOutput {
    public void export(DistanceMatrix matrix, Configuration config,
            Project project, File file) throws IOException {
        DistanceMatrixConfiguration distConfig =
            (DistanceMatrixConfiguration) config;
        PrintWriter writer = null;
        try {
            writer = new PrintWriter(file);

            TInvariants allTinvariants = project.getResult(
                TInvariantTool.class, new TInvariantsConfiguration());
            List<TInvariant> tinvariants = new ArrayList<TInvariant>(
                distConfig.hasTrivialTInvariants() ? allTinvariants :
                    allTinvariants.nonTrivialTInvariants());

            writer.println("distanceFunc " + getNormalizedString(distConfig.getDistanceFunctionName()));
            writer.println(tinvariants.size());

            for (int i = 0; i < matrix.size(); i++) {
                String name = "Inv" + tinvariants.get(i).id();
                if (name.length() > 9)
                    name = name.substring(0, 9);
                writer.printf("%-9s ", name);

                for (int j = 0; j < matrix.size(); j++)
                    writer.printf("%f ", matrix.get(i, j));

                writer.println();
            }
        }
        finally {
            if (writer != null)
                writer.close();
        }
    }

    public static String getNormalizedString(String s) {
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < s.length(); i++) {
            if (Character.isWhitespace(s.charAt(i))) {
                buf.append('_');
            } else {
                buf.append(s.charAt(i));
            }
        }
        return buf.toString();
    }
}
