/*
 *
 *  This file ist part of the software MonaLisa.
 *  MonaLisa is free software, dependend on non-free software. For more information read LICENCE and README.
 *
 *  (c) Molekulare Bioinformatik, Goethe University Frankfurt, Frankfurt am Main, Germany
 *
 */

package monalisa.data.pn;

import java.util.List;

/**
 * A place in a Petri net.
 * @author Konrad Rudolph
 */
public final class Place extends UniquePetriNetEntity implements Comparable<Place> {
    private static final long serialVersionUID = 3704844653776835796L;
    
    private PetriNet container;
    
    /**
     * Should be used only during deserialization.
     */
    @Deprecated
    public Place() { super(-1); }
    
    public Place(Place other) {
        super(other);
        // Does not belong to a Petri net, do not copy container.
    }

    public Place(int id) {
        super(id);
    }
    
    public void setContainer(PetriNet container) {
        this.container = container;
    }
    
    public PetriNet container() {
        return container;
    }

    public List<Transition> outputs() {
        return container.getTransitionsFor(this);
    }
    
    public List<Transition> inputs() {
        return container.getInputTransitionsFor(this);
    }

    @Override
    public int compareTo(Place o) {
        return id() - o.id();
    }
    
    @Override
    public String toString() {
        return "P" + id();
    }
}
