/*
 *
 *  This file ist part of the software MonaLisa.
 *  MonaLisa is free software, dependend on non-free software. For more information read LICENCE and README.
 *
 *  (c) Molekulare Bioinformatik, Goethe University Frankfurt, Frankfurt am Main, Germany
 *
 */

package monalisa.tools.cluster;

/*
 * Die Klasse NJ.java implementiert das phylogenetische Clusterverfahren
 * Neighbor Joining.
 * @author belau
 */
public class NJ {
    int cn; // Clusteranzahl innerhalb eines Clusters
    int leftCn; // Clusteranzahl innerhalb des linken Kindes
    int rigthCn; // Clusteranzahl innerhalb des rechten Kindes
    int n; // Anzahl der momentanen Cluster
    int m = 0; // Anzahl der vereinigten Cluster; l-m= Anzahl der momentanen
    // Blätteranzahl
    float dmin; // Dmin: minimale paarweise Distanz
    int imin; // Dmin: Index i
    int jmin; // Dmin: Index j
    float erg; // Ergebnis der Distanzberechnung zwischen 2 Clustern
    Cluster[] cluster; // array von Cluster-Objekten
    String name; // Clusterlabel
    String member; // T-Inv. innerhalb eines Clusters
    float faktor;
    float[] r; // mittlerer Abstand zu allen anderen Clustern

    /* Konstruktor der Klasse NJ */
    public NJ(float[][] d, float[][] e, String[] label) {

        int l = d.length - 1; // Anzahl der Blätter
        cluster = new Cluster[2 * l]; // Initialisierung des Cluster-arrays

        /* Initialisierung & Einlesen der Blätterknoten */
        for (int i = 1; i <= l; i++) {
            name = label[i - 1];
            member = name + ",";
            cluster[i] =
                new Cluster(member, name, i, 1, 0, 0, 0, null, 0, null, 0,
                    1.0f, 0, 0, 0);

        }
        n = l;
        int numClust = l;
        r = new float[numClust + 1];

        /* Berechnung von dmin */
        while (numClust > 2) {

            // Berechnung von r[]
            for (int i = 1; i <= d.length - 1; i++) {
                r[i] = 0;
                for (int j = 1; j <= d.length - 1; j++) {
                    r[i] += d[i][j];
                }
            }

            dmin = -1;
            int j = 1;
            faktor = (float) (1.0 / (numClust - 2));
            for (int i = 2; i <= l; i++) {
                if (d[i][j] != 0) {
                    for (j = 1; j < i; j++) {
                        if (d[i][j] != 0) {
                            erg = ((r[i] + r[j]) * faktor) - d[i][j];
                            if (erg > dmin) {
                                dmin = erg;
                                imin = i;
                                jmin = j;
                            }
                        }
                    }
                }
            }
            join(r, imin, jmin, d, faktor);
            numClust--;
            update(d, e);
        }// while

        // Vereinigung der letzten beiden Cluster
        int j = 1;
        for (int i = 2; i <= l; i++) {
            for (j = 1; j < i; j++) {
                if (d[i][j] != 0) {
                    erg = d[i][j];
                    imin = i;
                    jmin = j;
                }
            }
        }
        joinF(imin, jmin, d);
    }// Konstruktor NJ

    /*
     * Methode zur Vereinigung der Cluster i und j mit minimaler paarweiser
     * Distanz zu neuem Cluster k
     */
    public void join(float[] r, int i, int j, float[][] d, float faktor) {

        ++n; // Erhoehung der Clusteranzahl

        /* Berechnung der Objektattribute des linken Kindes */
        Cluster left = cluster[(int) d[imin][0]];
        int leftCn = left.getCn();
        String leftName = left.getName();

        /* Berechnung der Objektattribute des rechten Kindes */
        Cluster rigth = cluster[(int) d[0][jmin]];
        int rigthCn = rigth.getCn();
        String rigthName = rigth.getName();

        int cn = leftCn + rigthCn;
        String name = leftName + ", " + rigthName;
        String member = leftName + ", " + rigthName;
        int mitte = 1;
        name = "N" + n;

        /*
         * Berechnung der Distanzen zwischen Eltern- und Kindclustern
         * (leftHeigth, rightHeigth)
         */
        float leftHeigth =
            (d[imin][jmin] + ((r[imin] - r[jmin]) * faktor)) * 0.5f;
        float rightHeigth = (d[imin][jmin] - leftHeigth);

        /*
         * negative Distanzen werden null gesetzt, entsprechend dem Vorgehen in
         * phylip
         */
        if (leftHeigth < 0) {
            leftHeigth = 0.0f;
        }
        if (rightHeigth < 0) {
            rightHeigth = 0.0f;
        }

        /* Deklaration und Initialisierung des neuen Clusters k (innerer Knoten) */
        cluster[n] =
            new Cluster(member, name, n, cn, leftCn, rigthCn, dmin, left,
                leftHeigth, rigth, rightHeigth, mitte, 0, 0, 0);

    }// join

    /* Methode zur Neuberechnung der Distanzmatrix */
    public void update(float[][] d, float[][] e) {

        if (imin < jmin) { // Voraussetzung für update imin < jmin
            // Empty!
        } else {
            int i = imin;
            imin = jmin;
            jmin = i;
        }
        for (int i = 1; i <= d.length - 1; i++) {

            /*
             * Berechnung der Distanz zwischen dem neuen Cluster k und allen
             * anderen Clustern
             */
            if ((i != imin) && (i != jmin) && (d[imin][i] != 0)) {
                e[imin][i] = (d[imin][i] + d[jmin][i] - d[imin][jmin]) * 0.5f;
            } else {
                e[imin][i] = 0;
            }
        }

        /* Kopieren von e in d + Symmetrierung der Distanzmatrix */
        for (int i = 1; i <= d.length - 1; i++) {
            d[i][imin] = e[imin][i];
            d[imin][i] = d[i][imin];
        }

        /* Loeschen der Reihe und Spalte von jmin */
        for (int i = 1; i <= d.length - 1; i++) {
            d[i][jmin] = 0;
            d[jmin][i] = d[i][jmin];
        }

        /* Update der Beschriftung */
        d[imin][0] = n;
        d[0][imin] = n;
    }// update

    // Vereinigung der letzten beiden Cluster
    public void joinF(int i, int j, float[][] d) {

        ++n; // Erhoehung der Clusteranzahl

        /* Berechnung der Objektattribute des linken Kindes */
        Cluster left = cluster[(int) d[imin][0]];
        int leftCn = left.getCn();
        int leftIndex = left.getIndex();
        String leftName = left.getName();

        /* Berechnung der Objektattribute des rechten Kindes */
        Cluster rigth = cluster[(int) d[0][jmin]];
        int rigthCn = rigth.getCn();
        int rigthIndex = rigth.getIndex();
        String rigthName = rigth.getName();

        int cn = leftCn + rigthCn;
        String name = leftName + ", " + rigthName;
        String member = leftName + ", " + rigthName;
        int mitte = 1;
        name = "N" + n;

        Cluster last = cluster[n - 1];
        int lastIndex = last.getIndex();

        if (leftIndex == lastIndex) {
            float leftHeigth = 0f;
            float rightHeigth = d[imin][jmin];

            /*
             * negative Distanzen werden null gesetzt, entsprechend dem Vorgehen
             * in phylip
             */
            if (leftHeigth < 0) {
                leftHeigth = 0.0f;
            }
            if (rightHeigth < 0) {
                rightHeigth = 0.0f;
            }

            /*
             * Deklaration und Initialisierung des neuen Clusters k (innerer
             * Knoten)
             */
            cluster[n] =
                new Cluster(member, name, n, cn, leftCn, rigthCn, dmin, left,
                    leftHeigth, rigth, rightHeigth, mitte, 0, 0, 0);

        } else if (rigthIndex == lastIndex) {
            float rightHeigth = 0f;
            float leftHeigth = d[imin][jmin];

            /*
             * negative Distanzen werden null gesetzt, entsprechend dem Vorgehen
             * in phylip
             */
            if (leftHeigth < 0) {
                leftHeigth = 0.0f;
            }
            if (rightHeigth < 0) {
                rightHeigth = 0.0f;
            }

            /*
             * Deklaration und Initialisierung des neuen Clusters k (innerer
             * Knoten)
             */
            cluster[n] =
                new Cluster(member, name, n, cn, leftCn, rigthCn, dmin, left,
                    leftHeigth, rigth, rightHeigth, mitte, 0, 0, 0);
        }
    }// join

    /* Methode zur Uebergabe des Wurzel-Clusters */
    public Cluster getWurzel() {
        return cluster[n];
    }// getWurzel

}// class NJ
