package monalisa.tools.cluster;

import java.util.*;

/*
 * Die Klasse Validation.java implementiert die internen Validierungsmaße Dunn-,
 * Silhouetten-, Davies-Bouldin- und C-Index.
 * @author belau
 */
public class Validation {

    float sil; // SilhouetteIndex
    float cIndex; // C-Index
    float db; // Davies-Bouldin-Index (Zwischenergebnis)
    int indexZentrum; // Index des Clusterzentrums

    /** Creates a new instance of Validation */
    public Validation() {}

    /** Methode zur Berechnung der internen Validierungsmaße */
    public void validate(float matrix[][], String[] label, List<Leaves> groups) {
        /**
         * C-Index: die Distanzmatrix wird in einen array ueberfuehrt, welcher
         * der Groeße nach sortiert wird.
         */
        int size;
        size = matrix.length - 1;
        float[] arrayC;
        arrayC = new float[((size * size) - size) / 2];
        int zaehler = 0;

        for (int i = 2; i < matrix.length; i++) {
            for (int j = 1; j < i; j++) {
                arrayC[zaehler] = matrix[i][j];
                ++zaehler;
            }
        }

        Arrays.sort(arrayC);

        // die Inv-Ids (lmember) werden in die entsprechenden Indizes der matrix
        // ueberfuehrt um den Zugriff auf die paarweisen Distanzen zu
        // ermoeglichen
        int[] label2;
        label2 = new int[label.length + 1];

        for (int j = 0; j < label.length; j++) {
            String name = label[j];
            int idInv = Integer.parseInt(name.substring(3, name.length()));
            label2[j + 1] = idInv;
        }

        for (int i = 0; i < groups.size(); i++) {
            Leaves l = groups.get(i);
            Vector<Integer> member = l.lmember;
            for (int j = 0; j < member.size(); j++) {
                int InvId = (Integer) member.get(j);

                // Suche in matrix nach entsprechendem Index:
                int newId = binSearchIt(label2, InvId); // InvId wird durch
                // MatrixIndex ersetzt
                member.setElementAt(newId, j);
            }
        }

        /* intra-Cluster-Vergleich */

        float dist; // paarweise Distanz
        int index; // Anzahl paarweiser Distanzen innerhalb eines clusters

        float distD; // Dunn-Index: max. Distanz zwischen Objekten eines
        // Clusters
        float distC; // C-Index: Summe der paarweisen Distanzen innerhalb eines
        // Clusters

        float distS; // Silhouetten-Index: ai = mittlere Distanz zwischen i und
        // allen anderen Objketen des selben Clusters
        float distSSum; // Silhouetten-Index: Summe der paarweisen Distanzen
        // innerhalb eines Clusters
        float distSMin;
        /**
         * minimale mittlere Distanz innerahlb eines Clusters -> Clusterzentrum
         * wenn mehrere Inv. mit kleinster Dist, dann wird erste Inv als
         * Clusterentrum genommen
         */
        float[][] resSil; // Shilouetten-Index: Ergebnis-array: Inv-Index; ai;
        // bi
        resSil = new float[matrix.length][3];

        Vector<Float> resDBvar; // Zwischenergebnisse fuer DB-index: mittlere
                                // Distanz
        // aller Objekte zu Clusterzentrum
        resDBvar = new Vector<Float>();
        Vector<Integer> resDBind; // Zwischenergebnisse fuer DB-index: Index
        // Clusterzentrum
        resDBind = new Vector<Integer>();

        distD = 0;
        distS = 0;
        distSMin = 1000;
        distC = 0;
        index = 0;
        // int start = 0; // startIndex fuer resSil

        for (int i = 0; i < groups.size(); i++) { // jedes Blaettercluster
            Leaves l = (Leaves) groups.get(i);
            Vector<Integer> member = l.lmember;
            distSMin = 1000;

            for (int j = 0; j < member.size(); j++) { // interner Vergleich:
                // jedes Element mit
                // jedem
                int indexZ = member.get(j);
                distS = 0;
                distSSum = 0;

                for (int k = 0; k < member.size(); k++) {
                    if (k == j) {

                    } else {
                        int indexS = (Integer) member.get(k);
                        dist = matrix[indexZ][indexS];
                        distSSum += dist;

                        if (k >= j + 1) {
                            ++index;
                            if (dist > distD) {
                                distD = dist;
                            }
                            distC += dist;
                        }
                    }// else
                }// for

                if (member.size() > 1) { // mehr als ein Objekt in einem Cluster

                    /** Silhouetten-Index: Abspeichern der Zwischenergebnisse */
                    distS = distSSum / (member.size() - 1);
                    resSil[indexZ][0] = indexZ;
                    resSil[indexZ][1] = distS;

                    /** Davies-Bouldin-Index: Berechnung des Clusterzentrums */
                    if (distS < distSMin) {
                        distSMin = distS;
                        indexZentrum = indexZ;
                    }

                } else { // ein Objekt in einem Cluster

                    /** Silhouetten-Index: Abspeichern der Zwischenergebnisse */
                    resSil[indexZ][0] = indexZ;
                    resSil[indexZ][1] = 0;

                    /** Davies-Bouldin-Index: Berechnung des Clusterzentrums */
                    distSMin = 0;
                    indexZentrum = indexZ;
                }

            }

            /**
             * Davies-Bouldin-Index: mittlere Distanz alle Clusterobjekte zu
             * Cluterzentrum
             */
            distSMin = distSMin * (member.size() - 1) / member.size();
            resDBvar.add(distSMin);
            resDBind.add(indexZentrum);

        }// fuer jedes cluster

        /** Berechnung des C-Index */
        float cMax; // C-Index: Summe der n groeßten Distanzen
        float cMin; // C-Index: Summe der n kleinsten Distanzen
        int lastIndex = arrayC.length - 1;
        float resMax = 0;
        float resMin = 0;

        // Summe der n groeßten Werte
        for (int a = 0; a < index; a++) {
            cMax = arrayC[lastIndex - a];
            resMax += cMax;
        }

        // Summe der n kleinsten Werte
        for (int a = 0; a < index; a++) {
            cMin = arrayC[a];
            resMin += cMin;
        }
        // C-Index: Endresultat
        cIndex = (distC - resMin) / (resMax - resMin);

        /** Berechnung des Davies-Bouldin-Index */

        int zentrumI; // Index Clusterzentrum i
        int zentrumJ; // Index Clusterzentrum j
        float distI; // Varianz innerhalb Cluster i
        float distJ; // Varianz innerhalb Cluster j
        float sumVar; // Summe der Varianzen
        float distZentren; // Distanz zw. Clusterzentrum i und j
        float resultDB; // =sumVar/distZentren
        float resultDBMax; // max. resultDB
        // int zentrumJMax = 0; // Index Clusterzentrum (max.)

        float[] resDB; // Result-array Davies-Bouldin-Index
        resDB = new float[resDBvar.size()];

        // Initialisierung resDB
        for (int k = 0; k < resDB.length; k++) {
            resDB[k] = 0;
        }

        for (int i = 0; i < resDBvar.size(); i++) {
            resultDBMax = 0;
            distZentren = 0;
            sumVar = 0;
            ;
            zentrumI = (Integer) resDBind.get(i);
            distI = (Float) resDBvar.get(i);
            for (int j = 0; j < resDBvar.size(); j++) {
                if (j == i) {

                } else {
                    zentrumJ = (Integer) resDBind.get(j);
                    distJ = (Float) resDBvar.get(j);
                    distZentren = matrix[zentrumI][zentrumJ];
                    sumVar = distI + distJ;
                    resultDB = sumVar / distZentren;

                    if (resultDB > resultDBMax) {
                        resultDBMax = resultDB;
                        // zentrumJMax = zentrumJ;
                    }
                }
            }
            resDB[i] = resultDBMax;

        }

        float dbFinal;
        for (int k = 0; k < resDB.length; k++) {

            db += resDB[k];

        }
        dbFinal = db / resDB.length;

        /* inter-Cluster-Vergleich */

        float distInter; // paarweise Distanz zw. Objekten versch. Cluster
        int indexzInter; // fuer dieses Elemen werden alle paarweisen Distanzen
        // mit allen Elementen aus den anderen Clustern
        // gebildet
        float distDInter; // DunnIndex: min. Distanz zw. Objekten versch.
        // Cluster
        float distSInter; // Silhouetten-Index: mittelere Distanz zwischen i und
        // allen anderen Objekten der benachbarten Cluster
        float distSInterMin; // Silhouetten-Index: bi: mittelere Distanz
        // zwischen i und allen anderen Objekten des
        // naechsten benachbarten Clusters

        distInter = 0;
        distDInter = 1000;
        distSInterMin = 1000;

        for (int a = 0; a < groups.size(); a++) { // jedes Blaettercluster
            Leaves l = (Leaves) groups.get(a);
            Vector<Integer> member = l.lmember;

            for (int b = 0; b < member.size(); b++) {
                indexzInter = (Integer) member.get(b); // fuer dieses Elemen
                // werden alle
                // paarweisen Dist mit
                // allen Elementen aus
                // den anderen Clustern
                // gebildet
                distSInterMin = 1000;

                for (int c = 0; c < groups.size(); c++) { // Nachbarcluster

                    if (c == a) { // Vergleich mit jedem Nachbarcluster, außer
                        // dem gleichen

                    } else {
                        Leaves l2 = (Leaves) groups.get(c);
                        Vector<Integer> member2 = l2.lmember;
                        distSInter = 0;

                        for (int d = 0; d < member2.size(); d++) { // Elemente
                            // des
                            // Nachbarclusters

                            int indexs = (Integer) member2.get(d);
                            distInter = matrix[indexzInter][indexs];

                            // Silhouetten-Index
                            distSInter += distInter;

                            // Dunn-Index
                            if (distInter < distDInter) {
                                distDInter = distInter;
                            }
                        }
                        // Silhouetten-Index: mittelere Abstand zu dem
                        // benachbarten cluster
                        distSInter = distSInter / member2.size();

                        if (distSInter < distSInterMin) { // Abspeichern des
                            // minimalen
                            // mittleren
                            // Abstands
                            distSInterMin = distSInter;
                        }

                    }
                }
                resSil[indexzInter][2] = distSInterMin;
            }

        }

        /** Berechnung des Dunn-Index */
        float dunn = distDInter / distD;

        /** Berechnung des Silhouetten-Index */
        float result = 0;
        for (int p = 0; p < resSil.length; p++) { // interner Vergleich: jedes
            // Element mit jedem

            float max;

            float ai = resSil[p][1];
            float bi = resSil[p][2];
            if (ai > bi) {
                max = ai;
            } else {
                max = bi;
            }
            // Silhouettenwert wird nur berechnet fuer Elemente, die in ein
            // anderes Cluster eingeordet wurden -> ai>0
            if (ai == 0) {
                --size;
                bi = 0;
            }
            sil = (bi - ai) / max;
            result += sil;
        }

        sil = result / size;

        /** Ausgabe */
        System.out.println();
        System.out.println("Interne Clustervalidierung:");
        System.out.println();
        System.out.println("C-Index:" + "\t\t" + cIndex);
        System.out.println("Dunn-Index:" + "\t\t" + dunn);
        System.out.println("Silhouetten-Index:" + "\t" + sil);
        System.out.println("Davies-Bouldin-Index:" + "\t" + dbFinal);

    }// validate

    /** iterative binaere Suche */
    public static int binSearchIt(int[] werte, int ziel) {

        int m;
        int l = 0;
        int r = werte.length - 1;

        while (l <= r) {
            m = (r + l) / 2;

            if (ziel < werte[m]) {
                r = m - 1;
            } else if (ziel > werte[m]) {
                l = m + 1;
            } else {
                return m;
            }
        }
        return -1;

    }// binSearchIt

}// class Validation