/*
 *
 *  This file ist part of the software MonaLisa.
 *  MonaLisa is free software, dependend on non-free software. For more information read LICENCE and README.
 *
 *  (c) Molekulare Bioinformatik, Goethe University Frankfurt, Frankfurt am Main, Germany
 *
 */

package monalisa.tools.knockout;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import monalisa.data.Pair;
import monalisa.data.pn.PetriNet;
import monalisa.data.pn.Place;
import monalisa.data.pn.Transition;
import monalisa.data.pn.UniquePetriNetEntity;

public class DoublePlaceKnockout extends KnockoutAlgorithm {
    private List<Pair<Place, Place>> pairs
        = new ArrayList<Pair<Place, Place>>();
    private Iterator<Pair<Place, Place>> pairIterator;
    private List<UniquePetriNetEntity> currentKnockouts;
    
    DoublePlaceKnockout(PetriNet pn) {
        super(pn);
        
        List<Place> places = new ArrayList<Place>();
        for (Place p : pn.places())
            places.add(p);
        for (int i = 0; i < places.size(); i++)
            for(int j = i + 1; j < places.size(); j++)
                pairs.add(Pair.of(places.get(i), places.get(j)));
        
        pairIterator = pairs.iterator();
    }

    @Override
    protected PetriNet getNextKnockOutNetwork() {
        Pair<Place, Place> p = pairIterator.next();
        PetriNet copy = new PetriNet(getPetriNet());

        currentKnockouts = new ArrayList<UniquePetriNetEntity>();
        currentKnockouts.add(p.first());
        currentKnockouts.add(p.second());
        
        for (Transition t : p.first().outputs())
            copy.removeTransition(t);

        for (Transition t : p.first().inputs())
            copy.removeTransition(t);

        for (Transition t : p.second().outputs())
            copy.removeTransition(t);

        for (Transition t : p.second().inputs())
            copy.removeTransition(t);

        copy.removePlace(p.first());
        copy.removePlace(p.second());
        return copy;
    }

    @Override
    protected boolean hasNextKnockOutNetwork() {
        return pairIterator.hasNext();
    }
    
    protected List<UniquePetriNetEntity> getKnockoutEntities() {
        return currentKnockouts;
    }

    @Override
    protected int getTotalKnockouts() {
        return pairs.size();
    }
}
