/*
 *
 *  This file ist part of the software MonaLisa.
 *  MonaLisa is free software, dependend on non-free software. For more information read LICENCE and README.
 *
 *  (c) Molekulare Bioinformatik, Goethe University Frankfurt, Frankfurt am Main, Germany
 *
 */

package monalisa.tools.mcts;


import java.util.*;

import monalisa.data.pn.TInvariant;
import monalisa.data.pn.Transition;

public class InvariantStatistics {

    public static int[][] calculateMatrixTInvariant2TransitionOccurrence(
            List<TInvariant> invariants, 
            List<Transition> transitions) {
        int[][] matrix = new int[invariants.size()][transitions.size()];
        for (int indexInvariant = 0; indexInvariant < invariants.size(); indexInvariant++) {
            TInvariant inv = invariants.get(indexInvariant);
            List<Integer> iv = inv.asVector(transitions);
            for (int i = 0; i < iv.size(); i++) {
                matrix[indexInvariant][i] = iv.get(i);
            }
        }
        return matrix;
    }

//  public static void printOccurrenceMatrix(Vector<TInvariant> invariants,
//          Vector<Transition> transitions, PrintWriter out) {
//      int[][] matrix = calculateMatrixTInvariant2TransitionOccurrence(
//              invariants, transitions);
//
//      Vector<String> colNames = new Vector<String>();
//      Vector<String> rowNames = new Vector<String>();
//
//      for (int i = 0; i < invariants.size(); i++) {
//          rowNames.add(invariants.get(i).getName());
//      }
//
//      for (int i = 0; i < transitions.size(); i++) {
//          colNames.add(transitions.get(i).getName());
//      }
//      // utilities.MatrixVisualizer.printMatrix(out, matrix, rowNames,
//      // colNames);
//
//  }

//  public static Set<Transition> getNotCoveredTransitions(PetriNetInfo pni) {
//
//      Set<Transition> tSet = new HashSet<Transition>();
//      Vector<TInvariant> invariants = pni.getNonTrivialTInvariants();
//      Vector<Transition> transitions = pni.getSortedTransitionVector();
//
//      for (int indexTrans = 0; indexTrans < transitions.size(); indexTrans++) {
//          tSet.add(transitions.get(indexTrans));
//      }
//      for (int indexInv = 0; indexInv < invariants.size(); indexInv++) {
//          TInvariant inv = invariants.get(indexInv);
//          Set notNullOccTransitions = inv.getNotNullOccurrenceTransitions();
//          tSet.removeAll(notNullOccTransitions);
//      }
//
//      return tSet;
//  }

    public static List<TInvariant> getSupportMCTset(
            int[][] matrix,
            List<Transition> transitions, 
            List<TInvariant> invariants) {
        MCTSFactory mctsf = new MCTSFactory(matrix, transitions, invariants);
        return mctsf.mctsSupport();
    }

    public static List<TInvariant> getStrongMCTset(
            int[][] matrix,
            List<Transition> transitions, 
            List<TInvariant> invariants) {
        MCTSFactory mctsf = new MCTSFactory(matrix, transitions, invariants);
        return mctsf.strongMcts();
    }
}
