/*
 *
 *  This file ist part of the software MonaLisa.
 *  MonaLisa is free software, dependend on non-free software. For more information read LICENCE and README.
 *
 *  (c) Molekulare Bioinformatik, Goethe University Frankfurt, Frankfurt am Main, Germany
 *
 */

package monalisa.tools.topology;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.GroupLayout;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.TitledBorder;

import monalisa.Project;
import monalisa.data.Pair;
import monalisa.data.pn.Place;
import monalisa.data.pn.Transition;
import monalisa.resources.StringResources;
import monalisa.results.Configuration;
import monalisa.results.Topology;
import monalisa.results.TopologyConfiguration;
import monalisa.tools.AbstractTool;
import monalisa.tools.ErrorLog;
import monalisa.tools.Tool;

public class TopologyTool extends AbstractTool implements ActionListener {
    private static final String ACTION_CALCULATE = "CALCULATE";
    private static final String SELECT = "SELECT";
    private JPanel panel;
    private JCheckBox calculate;
    private JPanel toolPanel;
    private JRadioButton degreeDistribution;
    private JRadioButton avgCcDistribution;
    private Project project;
    
    @Override
    public boolean finishedState(Project project) {
        // TODO Auto-generated method stub
        return false;
    }
    
    @Override
    public JPanel getUI(Project project, StringResources strings) {
        this.project = project;
        if (panel == null) {
            exportButton.setActionCommand(EXPORT);
            exportButton.addActionListener(this);
            enableExport(project.hasResults(this));

            degreeDistribution = new JRadioButton(strings.get("DegreeDistribution"));
            degreeDistribution.setActionCommand(SELECT);
            degreeDistribution.addActionListener(this);
            
            avgCcDistribution = new JRadioButton(strings.get("AvgCcDistribution"));
            avgCcDistribution.setActionCommand(SELECT);
            avgCcDistribution.addActionListener(this);
                        
            ButtonGroup group = new ButtonGroup();
            group.add(degreeDistribution);
            group.add(avgCcDistribution);
            
            calculate = new JCheckBox(strings.get("Calculate"));
            calculate.setActionCommand(ACTION_CALCULATE);
            calculate.addActionListener(this);
            
            toolPanel = new JPanel();
            toolPanel.setBorder(BorderFactory.createTitledBorder(
                null, "",
                TitledBorder.DEFAULT_JUSTIFICATION,
                TitledBorder.DEFAULT_POSITION));
            GroupLayout toolLayout = new GroupLayout(toolPanel);
            toolPanel.setLayout(toolLayout);
            
            toolLayout.setHorizontalGroup(toolLayout.createParallelGroup()
                    .addComponent(degreeDistribution)
                    .addComponent(avgCcDistribution));
            
            toolLayout.setVerticalGroup(toolLayout.createSequentialGroup()
                    .addComponent(degreeDistribution)
                    .addComponent(avgCcDistribution));
            
            panel = new JPanel();
            GroupLayout layout = new GroupLayout(panel);
            panel.setLayout(layout);
            layout.setAutoCreateGaps(true);
            layout.setAutoCreateContainerGaps(true);
            
            layout.setHorizontalGroup(layout.createParallelGroup()
                .addComponent(toolPanel)
                .addComponent(calculate));
//                .addComponent(exportButton));
            
            layout.setVerticalGroup(layout.createSequentialGroup()
                .addComponent(toolPanel)
                .addComponent(calculate));
//                .addComponent(exportButton));
        }
        return panel;
    }

    @Override
    public boolean isActive() {
        return calculate.isSelected();
    }

    @Override
    public void run(Project project, ErrorLog log) {
        if (!degreeDistribution.isSelected() && !avgCcDistribution.isSelected())
            return;
        
        String result = "";
        String tool = "";      
      
        TopologyConfiguration config;
        if (degreeDistribution.isSelected()) {
            DegreeDistribution dd = new DegreeDistribution(project.getPetriNet());
            result = dd.computeRScript();
            tool = "DegreeDistributionR";
            config = new TopologyConfiguration(tool);
            addResult(config, new Topology(result, Topology.R));

            result = dd.computePlainText();
            tool = "DegreeDistributionPlain";
            config = new TopologyConfiguration(tool);
            addResult(config, new Topology(result, Topology.PLAIN));
        } else if (avgCcDistribution.isSelected()) {
            AvgClusteringCoefficients acc = new AvgClusteringCoefficients(project.getPetriNet(), getGraph("place"), getGraph("transition"));
            result = acc.compute();
            tool = "AvgCcDistribution";
            config = new TopologyConfiguration(tool);
            addResult(config, new Topology(result, Topology.R));
        }      

        enableExport(true);
    }
    
    private HashMap<String, List<String>> getGraph(String mode) {
        HashMap<String, List<String>> graph = new HashMap<String, List<String>>();
        
        if("place".equals(mode)) {          
            Collection<Transition> edges = project.getPetriNet().transitions();
            for(Transition e : edges){
                for(Place i : e.inputs()){
                    for(Place o : e.outputs()){
                        List<String> to = graph.get(i.getProperty("name"));
                        if(to == null){
                            to = new ArrayList<String>();
                            if(!((String) i.getProperty("name")).equals(((String) o.getProperty("name"))))
                                to.add(o.<String>getProperty("name"));
                        }else{
                            if(!to.contains(o.getProperty("name")) && !((String) i.getProperty("name")).equals(((String) o.getProperty("name")))){
                                to.add(o.<String>getProperty("name"));
                            }
                        }
                        graph.put(i.<String>getProperty("name"), to);
                    }
                }          
            }    
        } else if("transition".equals(mode)) {
            Collection<Place> edges = project.getPetriNet().places();                         
            for(Place e : edges){
                for(Transition i : e.inputs()){
                    for(Transition o : e.outputs()){
                        List<String> to = graph.get(i.getProperty("name"));
                        if(to == null){
                            to = new ArrayList<String>();
                            if(!((String) i.getProperty("name")).equals(((String) o.getProperty("name"))))
                                to.add(o.<String>getProperty("name"));
                        }else{
                            if(!to.contains(o.getProperty("name")) && !((String) i.getProperty("name")).equals(((String) o.getProperty("name")))){
                                to.add(o.<String>getProperty("name"));
                            }
                        }
                        graph.put(i.<String>getProperty("name"), to);
                    }
                } 
            }
        }
        
        return graph;
    }
    
    @Override
    public void saveSettings(Project p) {
        // TODO Auto-generated method stub
        
    }

    @Override
    public void setActive(boolean active) {
        // TODO Auto-generated method stub
        
    }

    @Override
    public void setActive(Configuration... configs) {
        // TODO Auto-generated method stub
        
    }

    @Override
    public List<Pair<Class<? extends Tool>, Configuration>> getRequirements() {
        return Collections.emptyList();
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        final String action = e.getActionCommand();

        if (action == ACTION_CALCULATE)
            fireActivityChanged(isActive());
        else if (action == SELECT)
            calculate.setSelected(true);
       else if(action == EXPORT) {
            List<Class<? extends Tool>> toolList = new ArrayList<Class<? extends Tool>>();
            toolList.add(TopologyTool.class);
            try {
                this.project.exportResults(toolList);
            } catch (IOException ex) {
                Logger.getLogger(TopologyTool.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }
}